//First of all we include all the libraries we will use for the code.
// In this case we only have the Adafruit Neopixel leds.

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>  // Required for 16 MHz Adafruit Trinket
#endif

// Here we define the pin were the NeoPixels will be attached to. In this case pin 6.
#define PIN 6

// We define the number of leds that the Neopixels has. In this case 32.
#define NUMPIXELS 32

// When setting up the NeoPixel library, we define how many pixels
// and which pin to use to send the signals. In order to configurate it,
// it is necessary to add the third parameter of the parenthesis, which
// controls the type of colors of the LEDs.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

// In this part we define the pins that will be reading the information from the different thumbtacks.
// Each of them represents one part of the robotic arm.
int boto1pin = A0, boto2pin = A1, boto3pin = A2, boto4pin = A3, boto5pin = A4;


// Here we define the pins of leds that wil attached to each thumbtacks, in order to indicate which robotic arm is being controlled.
int led1pin = 3, led2pin = 4, led3pin = 5, led4pin = 7, led5pinRed = 9, led5pinGreen = 10, led5pinBlue = 11;


// Here we define the angle limits of the robot arm.
float minrot0 = 360, maxrot0 = -360;
float minrot1 = 30, maxrot1 = 90;
float minrot2 = -120, maxrot2 = -60;
float minrot3 = -360, maxrot3 = 360;
float minrot4 = -60, maxrot4 = 60;


void setup() {

// These lines are specifically to support the Adafruit Trinket 5V 16 MHz.
#if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
  clock_prescale_set(clock_div_1);

#endif

  pixels.begin();  // INITIALIZE NeoPixel strip object (REQUIRED)

  // Here we establish the communication channel between Processing and Arduino.
  Serial.begin(115200);

  // Here we define the types of pins that each part of the arm is (INPUT/OUTPUT)
  pinMode(boto1pin, INPUT);
  pinMode(boto2pin, INPUT);
  pinMode(boto3pin, INPUT);
  pinMode(boto4pin, INPUT);
  pinMode(boto5pin, INPUT);

  //The same but with the LEDs
  pinMode(led1pin, OUTPUT);
  pinMode(led2pin, OUTPUT);
  pinMode(led3pin, OUTPUT);
  pinMode(led4pin, OUTPUT);
  pinMode(led5pinRed, OUTPUT);
  pinMode(led5pinGreen, OUTPUT);
  pinMode(led5pinBlue, OUTPUT);
}


void loop() {

  // First of all we declare that the LED ring is off.
  pixels.clear();

  // Then we declare new variables to store the value read of the sensor in real time.
  float sensorValue1 = analogRead(boto1pin);
  float sensorValue2 = analogRead(boto2pin);
  float sensorValue3 = analogRead(boto3pin);
  float sensorValue4 = analogRead(boto4pin);
  float sensorValue5 = analogRead(boto5pin);

  //We also declare state variables as buttons to define the actions to be performed when each arm is selected.
  // We initialize them to false.
  bool llegirBoto1 = false;
  bool llegirBoto2 = false;
  bool llegirBoto3 = false;
  bool llegirBoto4 = false;
  bool llegirBoto5 = false;

  //Here we establish in which specific parameters the Boolean variables of LlegirBoto will be true.
  //Serial.println(sensorValue1);
  if (sensorValue1 >= 70 && sensorValue2 <= 200 && sensorValue3 <= 15 && sensorValue4 == 0 && sensorValue5 == 0) {
    llegirBoto1 = true;

  } else {
    llegirBoto1 = false;
  }


  if (sensorValue2 >= 50 && sensorValue1 == 0 && sensorValue3 <= 220 && sensorValue4 <= 12 && sensorValue5 == 0) {
    llegirBoto2 = true;

  } else {
    llegirBoto2 = false;
  }


  if (sensorValue3 >= 50 && sensorValue1 == 0 && sensorValue2 == 0 && sensorValue4 <= 200 && sensorValue5 <= 11) {
    llegirBoto3 = true;

  } else {
    llegirBoto3 = false;
  }


  if (sensorValue4 >= 60 && sensorValue1 == 0 && sensorValue2 <= 0 && sensorValue3 == 0 && sensorValue5 <= 170) {
    llegirBoto4 = true;

  } else {
    llegirBoto4 = false;
  }


  if (sensorValue5 >= 60 && sensorValue1 <= 180 && sensorValue2 <= 11 && sensorValue3 <= 0 && sensorValue4 == 0) {
    llegirBoto5 = true;

  } else {
    llegirBoto5 = false;
  }

  // As the paint circle does not close at 360 degrees as it has a small gap to separate the 0V from the 5V,
  // we have defined to calibrate it, that the maximum number of degrees it reaches is 350.
  float angPotenciadorTotal = 350;


  // We set other variables that store the previous value that each arm has had.
  int anganterior1;
  int anganterior2;
  int anganterior3;
  int anganterior4;
  int anganterior5;


  //In this part we initialize to zero the variable that will transform the potential
  // that each arm will receive form the conductive paint and transform it into angles.
  float angPotenciadorLlegit1 = 0;
  float angPotenciadorLlegit2 = 0;
  float angPotenciadorLlegit3 = 0;
  float angPotenciadorLlegit4 = 0;
  float angPotenciadorLlegit5 = 0;


  //Here we calibrate each circle corresponding to one of the arms so that the voltages match the angles we want.
  if (sensorValue1 >= 335) {
    angPotenciadorLlegit1 = map(sensorValue1, 40, 215, 0, 120);
  } else {
    angPotenciadorLlegit1 = map(sensorValue1, 40, 200, 0, 120);
  }


  if (sensorValue2 >= 335) {
    angPotenciadorLlegit2 = map(sensorValue2, 40, 215, 0, 120);
  } else {
    angPotenciadorLlegit2 = map(sensorValue2, 40, 200, 0, 120);
  }


  if (sensorValue3 >= 335) {
    angPotenciadorLlegit3 = map(sensorValue3, 40, 215, 0, 120);
  } else {
    angPotenciadorLlegit3 = map(sensorValue3, 40, 200, 0, 120);
  }


  if (sensorValue4 >= 335) {
    angPotenciadorLlegit4 = map(sensorValue4, 40, 215, 0, 120);
  } else {
    angPotenciadorLlegit4 = map(sensorValue4, 40, 200, 0, 120);
  }


  if (sensorValue5 >= 335) {
    angPotenciadorLlegit5 = map(sensorValue5, 40, 215, 0, 120);
  } else {
    angPotenciadorLlegit5 = map(sensorValue5, 40, 200, 0, 120);
  }

  //Here we establish how many angles correspond to one led of the circle
  float angLed1 = angPotenciadorTotal / NUMPIXELS;
  float angLed2 = angPotenciadorTotal / NUMPIXELS;
  float angLed3 = angPotenciadorTotal / NUMPIXELS;
  float angLed4 = angPotenciadorTotal / NUMPIXELS;
  float angLed5 = angPotenciadorTotal / NUMPIXELS;


  //Here we make the conversion between the angle read by the variable
  //and the LEDs to be illuminated to represent that angle.
  int numLedsPintats1 = int(angPotenciadorLlegit1 / angLed1);
  int numLedsPintats2 = int(angPotenciadorLlegit2 / angLed2);
  int numLedsPintats3 = int(angPotenciadorLlegit3 / angLed3);
  int numLedsPintats4 = int(angPotenciadorLlegit4 / angLed4);
  int numLedsPintats5 = int(angPotenciadorLlegit5 / angLed5);


  //Here we define the actions that will occur when one of the arms is selected

  //This part of the code discusses an example of an arm that rotates up to 360 degrees.
  //This can only be done by arm 1 and 4.

  if (llegirBoto1 == true) {

    //Here we establish when it starts detecting angles and how they will be represented by the LED ring.
    if (sensorValue1 >= (anganterior1 / 2)) {

      //When the current starts to flow through the first pin, the red LED lights up.
      digitalWrite(led1pin, HIGH);

      //We illuminate in red the LEDs that we want in order to mark the angles we are defining.
      for (int i = 0; i < numLedsPintats1; i++) {
        pixels.setPixelColor(i, pixels.Color(255, 0, 0));
      }
      // Send the updated pixel colors to the hardware.
      pixels.show();

      //Here we update the variable that stores the previous angle to keep the position of the arm once another one is selected.
      anganterior1 = angPotenciadorLlegit1;
    }
    //Once the current no longer passes through the pin on the arm, we want the LEDs to turn off.
    //To do this, we illuminate them in black, which gives the same effect.
    digitalWrite(led1pin, LOW);

    for (int i = 0; i < numLedsPintats1; i++) {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }

    pixels.show();
  }


  //In this part, an example of an arm that only rotates up to 120 degrees is shown. This occurs in arm 2, 3 and 5.

  if (llegirBoto2 == true) {

    //Here we establish when it starts detecting angles and how they will be represented by the LED ring.
    if (sensorValue2 >= (anganterior2 / 2)) {

      //When the current starts to flow through the second pin, the blue LED lights up.
      digitalWrite(led2pin, HIGH);

      //Here we define that when the angle exceeds 120 degrees (limit angle) the LED ring will light up white
      // to indicate that the arm rotation limit has been exceeded.

      if (sensorValue2 >= 210) {
        for (int i = 0; i < 32; i++) {
          pixels.setPixelColor(i, pixels.Color(255, 255, 255));
        }
        pixels.show();  // Send the updated pixel colors to the hardware.

        //Here we define that when the angle is less than 120 degrees, that the led ring lights up blue (arm color).
      } else if (sensorValue2 <= 209) {
        for (int i = 0; i < numLedsPintats2; i++) {
          pixels.setPixelColor(i, pixels.Color(0, 0, 255));
        }
        pixels.show();  // Send the updated pixel colors to the hardware

        //Here we update the variable that stores the previous angle to keep the position of the arm once another one is selected.
        anganterior2 = angPotenciadorLlegit2;
      }
    }

    //Once the current no longer passes through the pin on the arm, we want the LEDs to turn off.
    digitalWrite(led2pin, LOW);

    for (int i = 0; i < 32; i++) {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }

    pixels.show();
  }

  // We do the same thing for the rest of the arms.

  if (llegirBoto3 == true) {

    if (sensorValue3 >= (anganterior3 / 2)) {
      //When the current starts to flow through the third pin, the green LED lights up.
      digitalWrite(led3pin, HIGH);

      if (sensorValue3 >= 210) {
        for (int i = 0; i < 32; i++) {
          pixels.setPixelColor(i, pixels.Color(255, 255, 255));
        }

        pixels.show();

      } else if (sensorValue3 <= 209) {
        for (int i = 0; i < numLedsPintats3; i++) {
          pixels.setPixelColor(i, pixels.Color(0, 255, 0));
        }

        pixels.show();
        anganterior3 = angPotenciadorLlegit3;
      }
    }

    digitalWrite(led3pin, LOW);

    for (int i = 0; i < 32; i++) {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }

    pixels.show();
  }


  if (llegirBoto4 == true) {

    if (sensorValue4 >= (anganterior4 / 2)) {
      //When the current starts to flow through the fourth pin, the yellow LED lights up.
      digitalWrite(led4pin, HIGH);

      for (int i = 0; i < numLedsPintats4; i++) {
        pixels.setPixelColor(i, pixels.Color(255, 100, 0));
      }

      pixels.show();
      anganterior4 = angPotenciadorLlegit4;
    }

    digitalWrite(led4pin, LOW);

    for (int i = 0; i < numLedsPintats4; i++) {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }

    pixels.show();
  }


  if (llegirBoto5 == true) {

    if (sensorValue5 >= (anganterior5 / 2)) {
      //When the current starts to flow through the fifth pin, the magenta RGB LED lights up.
      analogWrite(led5pinRed, 128);
      analogWrite(led5pinGreen, 0);
      analogWrite(led5pinBlue, 128);

      if (sensorValue5 >= 210) {
        for (int i = 0; i < 32; i++) {
          pixels.setPixelColor(i, pixels.Color(255, 255, 255));
        }

        pixels.show();

      } else if (sensorValue5 <= 209) {
        for (int i = 0; i < numLedsPintats5; i++) {
          pixels.setPixelColor(i, pixels.Color(128, 0, 128));
        }

        pixels.show();
        anganterior5 = angPotenciadorLlegit5;
      }
    }

    analogWrite(led5pinRed, 0);
    analogWrite(led5pinGreen, 0);
    analogWrite(led5pinBlue, 0);

    pixels.clear();

    for (int i = 0; i < numLedsPintats5; i++) {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }

    pixels.show();
  }

  //Serial.println(sensorValue2);
  // Create messages about the degrees to send it via serial to Processing

  Serial.println("J0_" + String(anganterior1));  // Send Angle data. Format is "A_123.456"
  Serial.println("J1_" + String(anganterior2));
  Serial.println("J2_" + String(anganterior3));
  Serial.println("J3_" + String(anganterior4));
  Serial.println("J4_" + String(anganterior5));
  Serial.flush();  // wait for messages to be sent

}
